==> ai_generation-1.1.0-alpha1.zip: ai_generation/src/Traits/ScaffoldGeneratorTrait.php <==
<?php

declare(strict_types=1);

namespace Drupal\ai_generation\Traits;

use Drupal\Core\Site\Settings;

/**
 * Declares a trait for create result content.
 */
trait ScaffoldGeneratorTrait {

  /**
   * Generate module/theme/config files based on AI JSON data.
   *
   * @param array $data
   *   The response from AI, must contain 'type', 'base_path', 'files'.
   *
   * @return array{written:int,skipped:int,paths:string[]}
   *   Stats and list of paths created.
   */
  protected function generateScaffold(array $data): array {

    if (empty($data)) {
      return [];
    }
    $type = strtolower((string) ($data['type'] ?? ''));
    $base = '';
    switch ($type) {
      case "module":
        $base = 'modules/custom';
        break;

      case "theme":
        $base = 'themes/custom';
        break;

      case "configuration":
        $base = Settings::get('config_sync_directory') ?: '../config/sync';
        break;

      case "recipe" || "frontend":
        $base = '../';
        break;
    }

    $written = 0;
    $files = $data['files'];
    foreach ($files as $file) {
      $rel = ltrim((string) ($file['path'] ?? ''), '/');
      $contents = (string) ($file['contents'] ?? '');

      if ($rel === '') {
        continue;
      }

      // AI already returns this info, just to be sure.
      if (str_starts_with($rel, 'modules/') || str_starts_with($rel, 'themes/')) {
        $target = $rel;
      }
      else {
        $target = rtrim($base, '/') . '/' . $rel;
      }

      $dir = dirname($target);
      if (!is_dir($dir)) {
        mkdir($dir, 0775, TRUE);
      }

      if (file_exists($target)) {
        continue;
      }
      file_put_contents($target, $contents);
      $written++;
    }

    return [
      'written' => $written,
      'type' => $type,
      'files' => implode("\n", array_column($data['files'], 'path')),
      'packageName' => $data['packageName'],
      'comments' => $data['comments'],
    ];
  }

}

==> ai_generation-1.2.0-alpha1.zip: ai_generation/src/Traits/ScaffoldGeneratorTrait.php <==
<?php

declare(strict_types=1);

namespace Drupal\ai_generation\Traits;

use Drupal\Core\Site\Settings;

/**
 * Declares a trait for create result content.
 */
trait ScaffoldGeneratorTrait {

  /**
   * Generate module/theme/config files based on AI JSON data.
   *
   * @param array $data
   *   The response from AI, must contain 'type', 'base_path', 'files'.
   * @param int $consumedTokens
   *   The total consumed tokens in the request.
   * @param string $time
   *   The total consumed tokens in the request.
   *
   * @return array{written:int,skipped:int,paths:string[]}
   *   Stats and list of paths created.
   */
  protected function generateScaffold(array $data, $consumedTokens, $time): array {

    if (empty($data)) {
      return [];
    }
    $type = strtolower((string) ($data['type'] ?? ''));
    $base = '';
    switch ($type) {
      case "module":
        $base = 'modules/custom';
        break;

      case "theme":
        $base = 'themes/custom';
        break;

      case "configuration":
        $base = Settings::get('config_sync_directory') ?: '../config/sync';
        break;

      case "recipe":
        $base = '../recipes';
        break;

      case "frontend":
        $base = '../apps';
        break;
    }

    $written = 0;
    $files = $data['files'];
    foreach ($files as $file) {
      $rel = ltrim((string) ($file['path'] ?? ''), '/');
      $contents = (string) ($file['contents'] ?? '');

      if ($rel === '') {
        continue;
      }

      $target = rtrim($base, '/') . '/' . $rel;
      $dir = dirname($target);
      if (!is_dir($dir)) {
        mkdir($dir, 0775, TRUE);
      }
      if (file_exists($target)) {
        continue;
      }
      file_put_contents($target, $contents);
      $written++;
    }

    return [
      'written' => $written,
      'type' => $type,
      'files' => implode("\n", array_column($data['files'], 'path')),
      'packageName' => $data['packageName'] ?? '',
      'comments' => $data['comments'],
      'tokens' => $consumedTokens,
      'time' => $time,
    ];
  }

}
